using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement the property page to pickup a Rep++ CardGroup
	/// </summary>
    //
    //********************************************************************************
	public class FormCgrpControlPropPage : System.Windows.Forms.Form	{
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container components = null;
		/// <summary>Field control being edited</summary>
		private CrdGrpControlUtil.ICrdGrpControl    m_cgrpControl;
		/// <summary>true if filling the combobox</summary>
		private bool                                m_bFilling;
        private System.Windows.Forms.Button         butOk;
        private System.Windows.Forms.Button         butCancel;
        private System.Windows.Forms.ComboBox       comboBoxCardName;
        private System.Windows.Forms.ComboBox       comboBoxGroupName;
        private System.Windows.Forms.Label          labelCardName;
        private System.Windows.Forms.Label          labelGroupName;
        private System.Windows.Forms.Button         butApply;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
		/// <param name="cgrpControl">      CardGroup control being edited</param>
        //
        //********************************************************************************
		public FormCgrpControlPropPage(CrdGrpControlUtil.ICrdGrpControl cgrpControl) {
			InitializeComponent();
		    m_cgrpControl = cgrpControl;
		    FillCardCombo();
		    SetApplyButtonState();
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
		public FormCgrpControlPropPage()	: this(null) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the object
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if GC</param>
        //
        //********************************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
				if (components != null)	{
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
        /// <summary>
        /// FillCardCombo:             Fills the card combo box with the list of groups
        /// </summary>
        //
        //********************************************************************************
        private void FillCardCombo() {
            RepPP.Application   app;
            CrdGrpFullName      cgrpFullName;
            Cards               crds;
            int                 iIndex;
            
            m_bFilling = true;
            app        = (m_cgrpControl == null) ? null : m_cgrpControl.Application;
            comboBoxCardName.Items.Clear();
            if (app != null) {
                cgrpFullName = m_cgrpControl.CardGroupFullName;
                crds         = app.Cards;
                foreach (Card crd in crds) {
                    if (String.Compare(crd.Name, 0, "$HIDE", 0, 5) != 0) {
                       comboBoxCardName.Items.Add(crd.Name);
                    }
                }
                if (cgrpFullName.CardName != String.Empty) {
                    iIndex = comboBoxCardName.Items.IndexOf(cgrpFullName.CardName);
                    comboBoxCardName.SelectedIndex = iIndex;
                }
            }
            m_bFilling = false;
            if (comboBoxCardName.SelectedIndex == -1 && comboBoxCardName.Items.Count != 0) {
                comboBoxCardName.SelectedIndex = 0;
            }
            FillGroupCombo();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillGroupCombo:             Fills the group combo box with the list of card groups
        /// </summary>
        //
        //********************************************************************************
        private void FillGroupCombo() {
            RepPP.Application   app;
            Card                crd;
            int                 iIndex;
            CrdGrpFullName      cgrpFullName;
            
            m_bFilling = true;
            app        = (m_cgrpControl == null) ? null : m_cgrpControl.Application;
            comboBoxGroupName.Items.Clear();
            if (app != null) {
                cgrpFullName = m_cgrpControl.CardGroupFullName;
                if (comboBoxCardName.SelectedIndex != -1) {
                    crd = app.Cards[comboBoxCardName.SelectedItem as string];
                    if (crd != null) {
                        FillCardGroupNames(crd.ChildCardGroups, comboBoxGroupName);
                        if (cgrpFullName.CardGroupName != String.Empty) {
                            iIndex                          = comboBoxGroupName.Items.IndexOf(cgrpFullName.CardGroupName);
                            comboBoxGroupName.SelectedIndex = iIndex;
                        }
                    }
                }
            }
            m_bFilling = false;
            if (comboBoxGroupName.SelectedIndex == -1 && comboBoxGroupName.Items.Count != 0) {
                comboBoxGroupName.SelectedIndex = 0;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillCardGroupNames:     Populates a combo box with the names of all CardGroups in
        ///                         a card
        /// </summary>
        /// <param name="cgrps">    Card Groups Collection</param>
        /// <param name="comboBox"> ComboBox Control</param>
        //
        //********************************************************************************
        internal void FillCardGroupNames(CardGroups cgrps, ComboBox comboBox) {
            foreach (CardGroup cgrp in cgrps) {
                if (String.Compare(cgrp.Name, 0, "$HIDE", 0, 5) != 0) {
                    comboBox.Items.Add(cgrp.Name);
                    FillCardGroupNames(cgrp.ChildCardGroups, comboBox);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetApplyButtonState:    Enable or disable the apply button depending on the context
        /// </summary>
        //
        //********************************************************************************
        private void SetApplyButtonState() {
            bool            bState = false;
            int             iIndex1;
            int             iIndex2;
            CrdGrpFullName  cgrpFullName;
            
            if (m_cgrpControl != null) {
                cgrpFullName    = m_cgrpControl.CardGroupFullName;
                iIndex1         = comboBoxCardName.SelectedIndex;
                iIndex2         = comboBoxGroupName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    if (comboBoxCardName.Items[iIndex1].ToString() != cgrpFullName.CardName ||
                        comboBoxGroupName.Items[iIndex2].ToString() != cgrpFullName.CardGroupName) {
                        bState = true;
                    }
                }
                butApply.Enabled = bState;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Apply:          Apply the change
        /// </summary>
        //
        //********************************************************************************
        private void Apply() {
            int             iIndex1;
            int             iIndex2;
            CrdGrpFullName  cgrpFullName;
            
            if (m_cgrpControl != null) {
                iIndex1 = comboBoxCardName.SelectedIndex;
                iIndex2 = comboBoxGroupName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    cgrpFullName = new CrdGrpFullName(comboBoxCardName.Items[iIndex1] as string,
                                                      comboBoxGroupName.Items[iIndex2] as string);
                    if (cgrpFullName.CardName != m_cgrpControl.CardGroupFullName.CardName ||
                        cgrpFullName.CardGroupName != m_cgrpControl.CardGroupFullName.CardGroupName) {
                        m_cgrpControl.CardGroupFullName  = cgrpFullName;
                        m_cgrpControl.TriggerComponentChange();
                    }
                }
            }
            butApply.Enabled = false;
        }

        //********************************************************************************
        //
        /// <summary>
        /// butOk_Click:            Called when the OK button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butOk_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// butApply_Click:         Called when the Apply button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butApply_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxCardName_SelectedIndexChanged:   Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxCardName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                FillGroupCombo();
                SetApplyButtonState();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxGroupName_SelectedIndexChanged:   Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxGroupName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                SetApplyButtonState();
            }
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.comboBoxCardName = new System.Windows.Forms.ComboBox();
            this.comboBoxGroupName = new System.Windows.Forms.ComboBox();
            this.butOk = new System.Windows.Forms.Button();
            this.butCancel = new System.Windows.Forms.Button();
            this.butApply = new System.Windows.Forms.Button();
            this.labelCardName = new System.Windows.Forms.Label();
            this.labelGroupName = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // comboBoxCardName
            // 
            this.comboBoxCardName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxCardName.Location = new System.Drawing.Point(56, 32);
            this.comboBoxCardName.Name = "comboBoxCardName";
            this.comboBoxCardName.Size = new System.Drawing.Size(224, 21);
            this.comboBoxCardName.TabIndex = 0;
            this.comboBoxCardName.SelectedIndexChanged += new System.EventHandler(this.comboBoxCardName_SelectedIndexChanged);
            // 
            // comboBoxGroupName
            // 
            this.comboBoxGroupName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxGroupName.Location = new System.Drawing.Point(56, 72);
            this.comboBoxGroupName.Name = "comboBoxGroupName";
            this.comboBoxGroupName.Size = new System.Drawing.Size(224, 21);
            this.comboBoxGroupName.TabIndex = 1;
            this.comboBoxGroupName.SelectedIndexChanged += new System.EventHandler(this.comboBoxGroupName_SelectedIndexChanged);
            // 
            // butOk
            // 
            this.butOk.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.butOk.Location = new System.Drawing.Point(32, 144);
            this.butOk.Name = "butOk";
            this.butOk.TabIndex = 2;
            this.butOk.Text = "OK";
            this.butOk.Click += new System.EventHandler(this.butOk_Click);
            // 
            // butCancel
            // 
            this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butCancel.Location = new System.Drawing.Point(120, 144);
            this.butCancel.Name = "butCancel";
            this.butCancel.TabIndex = 3;
            this.butCancel.Text = "Cancel";
            // 
            // butApply
            // 
            this.butApply.Location = new System.Drawing.Point(208, 144);
            this.butApply.Name = "butApply";
            this.butApply.TabIndex = 4;
            this.butApply.Text = "Apply";
            this.butApply.Click += new System.EventHandler(this.butApply_Click);
            // 
            // labelCardName
            // 
            this.labelCardName.Location = new System.Drawing.Point(8, 32);
            this.labelCardName.Name = "labelCardName";
            this.labelCardName.Size = new System.Drawing.Size(32, 23);
            this.labelCardName.TabIndex = 5;
            this.labelCardName.Text = "Card";
            // 
            // labelGroupName
            // 
            this.labelGroupName.Location = new System.Drawing.Point(8, 72);
            this.labelGroupName.Name = "labelGroupName";
            this.labelGroupName.Size = new System.Drawing.Size(40, 23);
            this.labelGroupName.TabIndex = 6;
            this.labelGroupName.Text = "Group";
            // 
            // FormCgrpControlPropPage
            // 
            this.AcceptButton = this.butOk;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.CancelButton = this.butCancel;
            this.ClientSize = new System.Drawing.Size(298, 182);
            this.Controls.Add(this.labelGroupName);
            this.Controls.Add(this.labelCardName);
            this.Controls.Add(this.butApply);
            this.Controls.Add(this.butCancel);
            this.Controls.Add(this.butOk);
            this.Controls.Add(this.comboBoxGroupName);
            this.Controls.Add(this.comboBoxCardName);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
            this.Name = "FormCgrpControlPropPage";
            this.Text = "Rep++ Card Group Property Page";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.ResumeLayout(false);

        }
		#endregion

	} // Class
} // Namespace
